<?php

namespace Drupal\batch_plugin;

use Drupal\batch_plugin\Logger\BatchPluginCronLog;

/**
 * Base class for processor plugins.
 */
abstract class QueueProcessorPluginBase extends ProcessorPluginBase implements QueueProcessorPluginInterface {

  const DEFAULT_CRON_EXPRESSION = '@daily';

  /**
   * The Queue ID.
   *
   * @var string
   */
  protected $queueId = '';

  /**
   * The logger.
   *
   * @var \Drupal\batch_plugin\Logger\BatchPluginCronLog
   */
  protected BatchPluginCronLog $logger;

  /**
   * {@inheritDoc}
   */
  public function getQueueId(): string {
    return $this->queueId;
  }

  /**
   * {@inheritDoc}
   */
  public function setQueueId($id): QueueProcessorPluginInterface {
    $this->queueId = $id;
    return $this;
  }

  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->logger = \Drupal::service('logger.batch_plugin_cron');
  }

  /**
   * {@inheritDoc}
   */
  public function breakLock(): void {
    $this->logger->breakLock($this->queueId);
  }

  /**
   * {@inheritDoc}
   */
  public function isQueueBuilding(): bool {
    $log = $this->logger->getLog($this->getQueueId());
    if (!empty($log)) {
      if ($log['running']) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritDoc}
   */
  public function addOperations(BatchPluginInterface $batch_plugin, array|\DrushBatchContext $context = []) {
    if (!isset($this->queueId)) {
      $this->queueId = $batch_plugin->getPluginDefinition()['queue_name'];
    }
    if (!isset($this->queueId)) {
      throw new \Exception('No Queue ID specified');
    }
    $this->batchPlugin = $batch_plugin;
    if ($this->isQueueBuilding()) {
      throw new \Exception('The plugin is already running');
    }
    $start_time = time();
    $this->logger->log($this->queueId, $start_time, 0, 1);
    parent::addOperations($batch_plugin, $context);
    $queue = \Drupal::queue($this->queueId);
    foreach ($this->operations as $item) {
      $queue->createItem($item);
    }
    $end_time = time();
    $this->logger->log($this->queueId, $start_time, $end_time, 0);
    return ProcessorPluginInterface::STATUS_OPERATIONS_ADDED;
  }

}
